/* === Copyright Notice ===
 *
 *
 *                  PureBasic source code file
 *
 *
 * This file is part of the PureBasic Software package. It may not
 * be distributed or published in source code or binary form without
 * the expressed permission by Fantaisie Software.
 *
 * By contributing modifications or additions to this file, you grant
 * Fantaisie Software the rights to use, modify and distribute your
 * work in the PureBasic package.
 *
 *
 * Copyright (C) 2000-2010 Fantaisie Software - all rights reserved
 *
 */

#ifndef PB_EVENT_H
#define PB_EVENT_H

typedef void (*EventDataFreeFunction)(void *Data);

#if !defined(PB_CARBON)
  // Gtk is UTF8 based, so we don't need 16bits strings routines at all
  #define NO_UNICODE_ALIASES
#endif

#define PB_InitEvent M_ThreadFunction(PB_InitEvent)

#define PB_USE_CFSTRING
#include <PureLibrary.h>
#include <Object/Object.h>


#if defined(PB_COCOA)
  #include <Cocoa/CocoaHeader.h>
  extern NSImage *PB_Event_DragImage;

#elif defined(PB_CARBON)
  #include <Carbon/Carbon.h>
  
  /* Properties ID, shared between Window and Gadget
   */
  #define PURE_Property 'PURE' 
  #define ID_Property   'ID'
  
  #define BoundsWidth(Bounds) (Bounds.right-Bounds.left)
  #define BoundsHeight(Bounds) (Bounds.bottom-Bounds.top)
  
  typedef int (*GetMenuWindowIDType)(MenuRef Menu);
  
  #define GetWindowIDProperty(Window, Property) GetWindowProperty(Window, PURE_Property, ID_Property, 4, NULL, Property)
  
  #define kVitualKeyF1 122
  #define kVitualKeyF2 120
  #define kVitualKeyF3 99
  #define kVitualKeyF4 118
  #define kVitualKeyF5 96
  #define kVitualKeyF6 97
  #define kVitualKeyF7 98
  #define kVitualKeyF8 100
  #define kVitualKeyF9 101
  #define kVitualKeyF10 109
  #define kVitualKeyF11 103
  #define kVitualKeyF12 111
  
  // #define kVitualKeyDEL 
  
  #define kCommandModifier (1 << 8)
  #define kShiftModifier   (1 << 9)
  #define kAltModifier     (1 << 11)
  #define kControlModifier (1 << 12)
  #define kNumericPadModifier (1 << 16)
   
#elif defined(LINUX) && !defined(PB_MACOS)
  #include <GtkBase/GtkBase.h>
  
#endif


#if defined(WINDOWS)

  #define GET_X_LPARAM(lp)	((int)(short)LOWORD(lp))
  #define GET_Y_LPARAM(lp)	((int)(short)HIWORD(lp))
  
  // Note: On Windows, every thread that creates a window must have a separate event loop,
  //   so we also have to keep thread-specific event queues to keep events separate.
  //   This is different on the other OS, as there we can have only one main event loop, so
  //   separation is not needed. The Window lib on Windows is already threadsafe otherwise.
  //
  //   Thread separation is done if this macro is defined (through the Object lib, so no 
  //   separate _THREAD versions needed)
  //
  // As some user uses threaded window, we keep it enabled
  //
  #define PB_EVENT_THREADEDQUEUES

  // Internal events
  #define PB_Event_SystrayInternal 12501

  // Public events
  #define PB_Event_SizeWindow     WM_SIZE
  #define PB_Event_Repaint        WM_PAINT
  #define PB_Event_Gadget         13100
  #define PB_Event_Menu           13101
  #define PB_Event_Systray        13102
  #define PB_Event_ActivateWindow 13104
  #define PB_Event_WindowDrop     13105
  #define PB_Event_GadgetDrop     13106
  //#define MSG_DirrectoryChange 13120 // for ExplorerListGadget
  #define PB_Event_MinimizeWindow  13107
  #define PB_Event_MaximizeWindow  13108
  #define PB_Event_RestoreWindow   13109
  #define PB_Event_Timer           13110 // do not just reuse WM_TIMER as event, as the WebGadget fires that too for example!
  #define PB_Event_RightClick      13111
  #define PB_Event_LeftClick       13112
  #define PB_Event_LeftDoubleClick 13113
  #define PB_Event_DeactivateWindow 13114
  #define PB_Event_TimeOut         0 // stay with 0, or it can cause hard to find bug while migrating code
  
  extern UINT  PB_Event_PostEventID;  // For PostEvent()
  extern HWND  PB_Event_PostEventWindow; // For PostEvent()

#else

  #define PB_Event_Menu              1
  #define PB_Event_CloseWindow       2
  #define PB_Event_Gadget            3
  #define PB_Event_MoveWindow        5
  #define PB_Event_Repaint           4
  #define PB_Event_SizeWindow        6
  #define PB_Event_ActivateWindow    7
  #define PB_Event_DeactivateWindow  8
  #define PB_Event_SysTray           9
  #define PB_Event_WindowDrop       10
  #define PB_Event_GadgetDrop       11
  #define PB_Event_MinimizeWindow   12
  #define PB_Event_MaximizeWindow   13
  #define PB_Event_RestoreWindow    14
  #define PB_Event_Timer            15
  #define PB_Event_RightClick       16
  #define PB_Event_LeftClick        17
  #define PB_Event_LeftDoubleClick  18
  #define PB_Event_TimeOut          0 // stay with 0, or it can cause hard to find bug while migrating code

  
  #define PB_Event_MouseButtonDown        20
  #define PB_Event_MouseButtonUp          21
  #define PB_Event_MouseButtonDoubleClick 22
  #define PB_Event_MouseButtonTripleClick 23
  #define PB_Event_MouseMove              24
  
  /* Event type
   */
  #define PB_EventType_Change           0x300
  #define PB_EventType_Focus            0x100
  #define PB_EventType_LeftClick        0x0
  #define PB_EventType_LeftDoubleClick  0x2
  #define PB_EventType_LostFocus        0x200
  #define PB_EventType_ReturnKey        0x501
  #define PB_EventType_RightClick       0x1
  #define PB_EventType_RightDoubleClick 0x3
  #define PB_EventType_DragStart        0x800
  
  #define PB_EventType_SizeItem  0xFFFE
  #define PB_EventType_CloseItem 0xFFFF
  
  
  /* Special type, for correct menu item placing on MacOS X
   */
  #define PB_Menu_Quit -1
  #define PB_Menu_Preferences -2
  #define PB_Menu_About -3

  // not used inside the Event lib, but in the Window lib
  extern int     PB_Event_ID;
  extern integer PB_Event_Type;
  extern integer PB_Event_ObjectID;
  extern integer PB_Event_WindowID;
  
  extern void *PB_Event_Data;
  extern EventDataFreeFunction PB_Event_FreeData;

#endif

#define PB_Event_User (1 << 16)+1

typedef struct PB_EventStruct
{
  struct PB_EventStruct *Next; // we only need a single link here
  
	int     ID;
  integer ObjectID;
  integer WindowID;
  integer Type;
  
  void *Data;
  EventDataFreeFunction FreeData;  
} PB_Event;


#ifdef PB_EVENT_THREADEDQUEUES

  // access goes through the Object lib  
  //
  typedef struct
  {
    PB_BlockAlloc *Allocator;
    PB_Event      *Head;
    PB_Event      *Tail;
    int            LockCount;
  } PB_EventGlobals;  
  
  extern integer PB_Event_Globals;
  #define PB_Event_GetGlobals() PB_EventGlobals *Globals = (PB_EventGlobals *)PB_Object_GetThreadMemory(PB_Event_Globals)
  
  extern PB_EventGlobals *PB_Event_MainThreadGlobals; // needed for PostEvent()
  
  #define PB_Event_LockCount  (Globals->LockCount)
  #define PB_Event_Allocator  (Globals->Allocator)
  #define PB_Event_Head       (Globals->Head)
  #define PB_Event_Tail       (Globals->Tail)

#else

  // no need for the object lib here
  #define PB_Event_GetGlobals()
  
  extern int            PB_Event_LockCount;
  extern PB_BlockAlloc *PB_Event_Allocator;
  extern PB_Event      *PB_Event_Head; // read here
  extern PB_Event      *PB_Event_Tail; // add here
  
#endif

#ifdef PB_CARBON
  extern GetMenuWindowIDType PB_Event_GetMenuWindowID;
#endif

//#define PB_Event_Add   M_ThreadFunction(PB_Event_Add)
//#define PB_Event_Get   M_ThreadFunction(PB_Event_Get)
//#define PB_Event_Init  M_ThreadFunction(PB_Event_Init)

// When using PB_Event_AddWithData, the caller of PB_Event_Get must ensure that
// the FreeData function is called when it received a non-zero free function pointer

void PB_Event_Add(int ID, integer ObjectID, integer WindowID, integer Type);
void PB_Event_AddWithData(int ID, integer ObjectID, integer WindowID, integer Type, void *Data, EventDataFreeFunction FreeData);
int  PB_Event_Get(integer *ObjectID, integer *WindowID, integer *Type, void **Data, EventDataFreeFunction *FreeData);
int  PB_Event_PeekID();
int  PB_Event_Peek(integer *ObjectID, integer *WindowID, integer *Type);
void PB_Event_Lock();
void PB_Event_Unlock();
void PB_Event_ClearAll(integer WindowID);
void PB_Event_Clear(int ID, integer ObjectID, integer WindowID, integer Type);


#endif
