/* === Copyright Notice ===
 *
 *
 *                  PureBasic source code file
 *
 *
 * This file is part of the PureBasic Software package. It may not
 * be distributed or published in source code or binary form without
 * the expressed permission by Fantaisie Software.
 *
 * By contributing modifications or additions to this file, you grant
 * Fantaisie Software the rights to use, modify and distribute your
 * work in the PureBasic package.
 *
 *
 * Copyright (C) 2000-2010 Fantaisie Software - all rights reserved
 *
 */

/* Gadget constants that are common to all OS
 */

// -----------------------------------------------------------
// CanvasGadget
// -----------------------------------------------------------

// EventType for this gadget
//
// Note: This gadget also supports the Focus, LostFocus, Left/Right Click and
//   Left/RightDoubleClick EventTypes, so make sure they do not overlap with these
//
#define PB_Canvas_FirstEventType 0x10000

#define PB_EventType_MouseEnter        (PB_Canvas_FirstEventType+1)
#define PB_EventType_MouseLeave        (PB_Canvas_FirstEventType+2)
#define PB_EventType_MouseMove         (PB_Canvas_FirstEventType+3)
#define PB_EventType_LeftButtonDown    (PB_Canvas_FirstEventType+4)
#define PB_EventType_LeftButtonUp      (PB_Canvas_FirstEventType+5)
#define PB_EventType_RightButtonDown   (PB_Canvas_FirstEventType+6)
#define PB_EventType_RightButtonUp     (PB_Canvas_FirstEventType+7)
#define PB_EventType_MiddleButtonDown  (PB_Canvas_FirstEventType+8)
#define PB_EventType_MiddleButtonUp    (PB_Canvas_FirstEventType+9)
#define PB_EventType_MouseWheel        (PB_Canvas_FirstEventType+10)
#define PB_EventType_KeyDown           (PB_Canvas_FirstEventType+11)
#define PB_EventType_KeyUp             (PB_Canvas_FirstEventType+12)
#define PB_EventType_Input             (PB_Canvas_FirstEventType+13)


// Flags
//
#define PB_Canvas_Border     1
#define PB_Canvas_ClipMouse  2
#define PB_Canvas_Keyboard   4
#define PB_Canvas_DrawFocus  8

// Get/SetGadgetAttribute
//
#define PB_Canvas_Image        1
#define PB_Canvas_MouseX       2
#define PB_Canvas_MouseY       3
#define PB_Canvas_Buttons      4
#define PB_Canvas_Key          5
#define PB_Canvas_Modifiers    6
#define PB_Canvas_Cursor       7
#define PB_Canvas_WheelDelta   8
#define PB_Canvas_Input        9
#define PB_Canvas_Clip         10
#define PB_Canvas_CustomCursor 11

// for PB_Canvas_Modifiers
//
#define PB_Canvas_Shift        (1 << 0)
#define PB_Canvas_Alt          (1 << 1)
#define PB_Canvas_Control      (1 << 2)
#define PB_Canvas_Command      (1 << 3)

// for PB_Canvas_Buttons
//
#define PB_Canvas_LeftButton    (1 << 0)
#define PB_Canvas_RightButton   (1 << 1)
#define PB_Canvas_MiddleButton  (1 << 2)

// For GadgetX/Y
#define PB_Gadget_ContainerCoordinate 0 // default
#define PB_Gadget_ScreenCoordinate    (1 << 0)
#define PB_Gadget_WindowCoordinate    (1 << 1)

// StringGadget() Attribute
#define PB_String_MaximumLength 1


// constants for PB_Canvas_Cursor
//
enum
{
  PB_Cursor_Default,   // default arrow cursor
  PB_Cursor_Cross,     // crosshair
  PB_Cursor_IBeam,     // I text cursor
  PB_Cursor_Hand,      // hand cursor
  PB_Cursor_Busy,      // hourglass/watch cursor
  PB_Cursor_Denied,    // slashed circle/ X cursor (not available in Cocoa)
  PB_Cursor_Arrows,    // arrows in all directions (not available in Cocoa/Carbon)
  PB_Cursor_LeftRight, // <-> arrow
  PB_Cursor_UpDown,    // up/down arrow  
  PB_Cursor_LeftUpDownRight, // diagonal arrows 1
  PB_Cursor_LeftDownUpRight, // diagonal arrows 2
  PB_Cursor_Invisible, // invisible cursor  
  
  PB_Cursor_Last = PB_Cursor_Invisible // just for internal use
};



