/* === Copyright Notice ===
 *
 *
 *                  PureBasic source code file
 *
 *
 * This file is part of the PureBasic Software package. It may not
 * be distributed or published in source code or binary form without
 * the expressed permission by Fantaisie Software.
 *
 * By contributing modifications or additions to this file, you grant
 * Fantaisie Software the rights to use, modify and distribute your
 * work in the PureBasic package.
 *
 *
 * Copyright (C) 2000-2010 Fantaisie Software - all rights reserved
 *
 */

#ifndef PB_SYSTEMBASE_H
#define PB_SYSTEMBASE_H

#include <PureLibrary.h>

M_SYSFUNCTION(void) SYS_FreeStructureStrings(char *Area, integer *StructureMap);
M_SYSFUNCTION(void) SYS_CopyStructure(char *Destination, integer Length, integer *StructureMap, const char *Source);
M_SYSFUNCTION(void) SYS_InitDynamicStructure(char *Address, integer *StructureMap);
M_SYSFUNCTION(integer) SYS_IsDynamicStructure(integer *StructureMap);
M_SYSFUNCTION(void) SYS_AdjustStructurePointers(char *Address, integer *StructureMap);

M_SYSFUNCTION(void *) SYS_AllocateMemory(integer Size);
M_SYSFUNCTION(void *) SYS_AllocateMemoryFast(integer Size);
M_SYSFUNCTION(void)   SYS_AllocateDynamicStructure(integer **Address, integer *StructureMap);
M_SYSFUNCTION(void *) SYS_ReAllocateMemory(void *OldMemory, integer OldSize, integer NewSize);
M_SYSFUNCTION(void)   SYS_FreeMemory(void *Memory);
M_SYSFUNCTION(void *) SYS_AllocateMemoryWithSize(integer Size);
M_SYSFUNCTION(void *) SYS_AllocateMemoryWithSizeNoClear(integer Size);
M_SYSFUNCTION(void *) SYS_ReAllocateMemoryWithSize(void *OldMemoryP, integer NewSize);
M_SYSFUNCTION(void *) SYS_ReAllocateMemoryWithSizeNoClear(void *OldMemoryP, integer NewSize);
M_SYSFUNCTION(void)   SYS_FreeMemoryWithSize(void *Memory);
M_SYSFUNCTION(void)   SYS_ClearStructure(char *Address, integer Length, integer *StructureMap);

M_SYSFUNCTION(void *) SYS_GetThreadedObject();

typedef void (M_PBVIRTUAL * ThreadCallbackType)(void *Data);
M_SYSFUNCTION(void) SYS_RegisterThreadCallback(ThreadCallbackType Callback, void *Data);

// Threaded Object stuff
extern integer PB_ThreadedObjectSize; // defined by the Compiler (available when threaded objects are used and in debug mode)
extern M_SYSFUNCTION(void *) SYS_GetThreadedObject();

// Structure items markers
#define ST_End         -1
#define ST_StaticArray -2
#define ST_Structure   -3
#define ST_Array       -4
#define ST_List        -5
#define ST_Map         -6
#define ST_MultiArray  -7



// Dynamic functions, nested objects in structures
struct PB_Array;
struct PB_List;
struct PB_ListObject;
struct PB_ListHeader;
struct PB_Map;

typedef struct PB_Array * (M_SYSVIRTUAL *NewArrayProc)     (integer ElementSize, integer NbElements, int Type, integer *StructureMap, struct PB_Array **Address);
typedef struct PB_Array * (M_SYSVIRTUAL *NewMultiArrayProc)(integer NbDimensions, integer ElementSize, int Type, integer *StructureMap, integer *Address);
typedef struct PB_List *  (M_PBVIRTUAL *NewListProc)       (integer ElementSize, struct PB_ListObject *Object, integer *StructureMap, int FirstElementType);
typedef struct PB_Map *   (M_PBVIRTUAL *NewMapProc)        (integer ElementSize, integer *StructureMap, struct PB_Map **Address, int HashSize);

typedef integer (M_PBVIRTUAL *CopyArrayProc) (const struct PB_Array *Array, struct PB_Array *DestinationArray, int Clear);
typedef integer (M_PBVIRTUAL *CopyListProc)  (const struct PB_List *List, struct PB_List *DestinationList, int Clear);
typedef integer (M_PBVIRTUAL *CopyMapProc)   (struct PB_Map *Map, struct PB_Map *DestinationMap, int Clear);

typedef void (M_PBVIRTUAL *FreeArrayProc) (struct PB_Array *Array);
typedef void (M_PBVIRTUAL *FreeListProc)  (struct PB_List *List);
typedef void (M_PBVIRTUAL *FreeMapProc)   (struct PB_Map *Map);

extern NewMultiArrayProc SYS_DynamicNewMultiArray;
extern NewArrayProc      SYS_DynamicNewArray;
extern NewListProc       SYS_DynamicNewList;
extern NewMapProc        SYS_DynamicNewMap;

extern CopyArrayProc SYS_DynamicCopyArray;
extern CopyListProc  SYS_DynamicCopyList;
extern CopyMapProc   SYS_DynamicCopyMap;

extern FreeArrayProc SYS_DynamicFreeArray;
extern FreeListProc  SYS_DynamicFreeList;
extern FreeMapProc   SYS_DynamicFreeMap;


#ifdef X64
  // Helper function to convert from PB FPU stack to AMD64 SSE convention
  M_SYSFUNCTION(float) SYS_ConvertFloatFPU();
  M_SYSFUNCTION(double) SYS_ConvertDoubleFPU();
#endif

#endif

